/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2001-2002 Vivid Solutions Inc.
 * Copyright (C) 2005-2006 Refractions Research Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation.
 * See the COPYING file for more information.
 *
 **********************************************************************/

#ifndef GEOS_PLANARGRAPH_H
#define GEOS_PLANARGRAPH_H

namespace geos {

/// Contains classes to implement a planar graph data structure.
namespace planargraph { // geos::planargraph

/// Planargraph algorithms
namespace algorithm { // geos::planargraph::algorithm

} // namespace geos::planargraph::algorithm
} // namespace geos::planargraph
} // namespace geos

//#include <geos/planargraph/GraphComponent.h>
//#include <geos/planargraph/DirectedEdgeStar.h>
//#include <geos/planargraph/Node.h>
//#include <geos/planargraph/Edge.h>
//#include <geos/planargraph/DirectedEdge.h>
#include <geos/planargraph/NodeMap.h>
#include <geos/planargraph/PlanarGraph.h>
//#include <geos/planargraph/Subgraph.h>
//#include <geos/planargraph/algorithm/ConnectedSubgraphFinder.h>

#endif

